"""Load-manager for downloading multi-value URLs

The Singleton "Loader" should be used for most interactions.

Note that there is a long associated with the loader
	OpenGLContext.debug.logs.loader_log
"""
from OpenGLContext.debug.logs import loader_log, DEBUG
import urllib, os

##loader_log.setLevel( DEBUG )

class _Loader( object ):
	"""(Singleton) Manager for downloading resources

	Is a generic class which provides download services
	which follow VRML97 semantics (multiple-URL definitions,
	with chaining to the first successful URL).

	The loader will attempt to use a local cache of files
	to prevent multiple downloading.
	"""
	def __init__(
		self,
	):
		"""Initialize the Loader"""
		self.cache = {}

	def __call__( self, url, baseURL=None ):
		"""Load the given multi-value url and call callbacks

		url -- vrml97-style url (multi-value string)
		baseURL -- optional base url from which items in url will
			be resolved.  protofunctions.root(node).baseURI will
			give you the baseURL normally used for the given node.

		raises IOError on failure
		returns (successfulURL, filename, open_file, headers) on success

		headers will be None for local files
		"""
		loader_log.info( "Loading: %s, %s", url, baseURL )
		if isinstance( url, (str, unicode) ):
			url = [url]
		file = None
		for u in url:
			# get the "absolute" url
			if baseURL:
				u = urllib.basejoin(baseURL, u )
			resolvedURL, file, filename, headers = self.get( u )
			if file is not None and filename is not None:
				break
		if not file or not filename:
			raise IOError( """Unable to download url %s"""%url)
		return ( resolvedURL, os.path.abspath(filename), file, headers )
	def get( self, url ):
		"""Retrieve the given single-value URL

		url -- single-value URL, which may be a local filename
			or any URL type supported by urllib

		returns (baseURL, file, filename, headers)
		"""
		headers = None
		if self.cache.has_key( url ):
			filename = self.cache.get( url )
			loader_log.debug( "cached: %s %s", url, filename )
			try:
				file = open( filename, 'rb')
			except (IOError,TypeError,ValueError):
				pass
		try:
			loader_log.debug( "load: %s", url )
			file = open( url, 'rb' )
			filename = url
			baseURL = urllib.pathname2url( filename )
		except (IOError,TypeError,ValueError), err:
			# try to download
			try:
				loader_log.debug( "download: %s", url )
				filename, headers = self.download( url )
				loader_log.debug( "downloaded to: %s", filename )
				file = open( filename, 'rb')
				baseURL = url
			except (IOError, TypeError, ValueError), err:
				return (None, None,None,None)
		self.cache[ url ] = filename
		self.cache[ baseURL ] = filename
		return baseURL, file, filename, headers
	def download( self, url ):
		"""Download the given url to local disk, return local filename"""
		filename, headers = urllib.urlretrieve(
			url,
		)
		return filename, headers

Loader = _Loader()
