"""VRML97 loader module for OpenGLContext

This module implements VRML97-parser handler for the
loader module.  The parser is provided by the
vrml.vrml97 module.  Of particular interest to the
end-developer is the standardPrototype function, which
allows you to register prototypes as standard features
for your VRML97 files.

By default, the standard prototype namespace is the
union of:
	vrml.vrml97.basenodes and
	OpenGLContext.scenegraph.basenodes

with prototypes from the second replacing those in the
first.

Note that there is a log associated with the loader
which is used by the VRML97 loader module:
	OpenGLContext.debug.logs.loader_log
"""
from vrml.vrml97 import basenamespaces, parser, linearise, nodetypes
from vrml import node, protofunctions
from OpenGLContext.scenegraph import basenodes
from OpenGLContext.debug.logs import loader_log, DEBUG
from vrml.vrml97 import parseprocessor
import urllib

##loader_log.setLevel( DEBUG )
filename_extensions = ['.wrl', '.wrl.gz', '.wrz', '.vrml', '.vrml.gz']

STANDARD_PROTOTYPES = basenamespaces.basePrototypes.copy()

def standardPrototype( prototype, key ):
	"""Make the given prototype available as a standard prototype

	What this means is that VRML97 files loaded
	with this module will be able to access the prototype
	without needing to declare a PROTO within the
	file.

	The name registered is the result of protofunctions.name
	for the prototype.
	"""
	name = protofunctions.name( prototype )
	STANDARD_PROTOTYPES[name] = prototype
	if name != key:
		loader_log.info( "Standard prototype %r is known by the key %r instead of it's prototype name", name, key )
	return name

### Update from the basenodes dictionary of OpenGLContext
for key,value in basenodes.__dict__.items():
	try:
		if issubclass(value, node.Node):
			name = standardPrototype( value, key )
	except TypeError:
		pass

_parser = parser.Parser( parser.grammar, "vrmlFile")

class VRML97Handler( object ):
	"""Handler for loading VRML97-encoded scenegraphs

	This is a load handler for the loader module which
	will be instantiated by the load or loads function
	in order to handle the result of downloading/opening
	a VRML97-encoded file.

	The prototypes argument is normally a pointer to
	the STANDARD_PROTOTYPES namespace, so that prototype
	registration during downloading will be available
	during parsing.
	"""
	def __init__( self, prototypes ):
		"""Initialise the file-handler

		prototypes -- prototype namespace provided by the
			vrml.protonamespace package
		"""
		self.prototypes = prototypes
	def __call__( self, baseURL, filename, file, *args, **named ):
		"""Load VRML97-encoded scenegraph from the file

		baseURL -- the URL from which the file was loaded
		filename -- the local filename for the file
		file -- open read-only file handle

		Returns a scenegraph or None
		"""
		# handle potential for a gzipped file...
		loader_log.debug( "VRML97 handler file: %s", filename)
		# Okay, file is an open file
		# check the type to see if we need to gunzip
		if isGzip( file ):
			loader_log.debug( "gunzip: %s", filename )
			file = gunzip( file )
		# Should check the file type,
		# just in case it's not really VRML, but that's
		# not really critical at the moment.
		
		# Get the data as in-memory string
		loader_log.debug( "read: start")
		data = file.read()
		loader_log.debug( "read: %s bytes", len(data))
		success, results, next = _parser.parse(
			data,
			processor = parseprocessor.ParseProcessor(
				basePrototypes = self.prototypes,
				baseURI = baseURL,
			)
		)
		if success and results:
			loader_log.debug( "parse complete")
			return results[1]
		elif not success:
			loader_log.warn( "parse of %s failed"%(baseURL))
			raise ValueError( """Parse failure for url %s"""%(baseURL))
		else:
			loader_log.warn( "parse of %s returned NULL document")
			raise ValueError( """NULL results for url %s"""%(baseURL))
		

def dumps( node ):
	"""Dump node's representation to a VRML97 string"""
	return linearise.Lineariser().linear( node )

def dump( node, file ):
	"""Dump node's representation to a VRML97-formatted file"""
	data = dumps( node )
	if isinstance( file, str ):
		file = open( file, 'w')
	file.write( data )
	file.close()
	return data

def load( url, baseURL = None ):
	"""Load the given URL as a VRML97 scenegraph

	url -- the URL (or list of URLs) from which to load
	baseURL -- optional base URL from which to determine
		relative URL values
	"""
	from OpenGLContext.loaders import loader
	result = loader.Loader( url, baseURL=baseURL )
	if not result:
		return result
	return VRML97Handler( STANDARD_PROTOTYPES )( *result )

def gunzip( file ):
	"""Get a gzip-aware file for the given file handle"""
	import gzip
	return gzip.GzipFile("","rb", 9, file)

def loads( data ):
	"""Load a VRML97-formatted string into a scenegraph"""
	value = _parser.parse(
		data,
		processor = parseprocessor.ParseProcessor(
			basePrototypes = STANDARD_PROTOTYPES,
			baseURI = "",
		)
	)[1][1] 
	loader_log.info( "parse complete: %s", str(value) )
	return value
	
def isGzip( file ):
	"""Determine whether the data is a gzip stream"""
	previous = file.tell()
	try:
		file.seek(0)
		magic = file.read(5)
		if magic[:3] == '\037\213\010':
			return 1
		elif magic[:5] == '#VRML':
			return 0
		else:
			return None
	finally:
		file.seek(previous)

