"""Mix-in class for contexts needing to control a viewplatform object"""
from OpenGLContext import interactivecontext, viewplatform, context
from OpenGL.GL import *
import math
class ViewPlatformMixin:
	"""Mix-in for Context classes providing ViewPlatform support

	The viewplatform module provides a ViewPlatform object
	which provides generic "camera" support for OpenGLContext.
	This mix-in provides Context classes with automatic support
	for instantiating and using these objects.  In particular,
	it overrides the Viewpoint customization point, dispatching
	the call to the platform's render method.

	In addition, the ViewPlatformMixin includes support for the
	view-platform-specific examine manager (which rotates the
	view platform about an object-space center).

	The getViewPlatform method performs the actual instantiation
	of the ViewPlatform, which allows a sub-classes to use
	different view-platform classes with minimal interference.

	Finally, the ViewPlatformMixin performs "default" registration
	of various event handlers using the setupDefaultEventCallbacks
	customization point.  These provide the default navigation
	controls for OpenGLContext Contexts.

	Attributes:
		platform -- the view platform instantiated by the
			context, or None if there is not yet a view
			platform instantiated (the platform is normally
			instantiated during the first rendering pass)
		initialPosition -- the initial view position used
			by the platform.
			See: ViewPlatform.setPosition
		initialOrientation -- the initial view orientation
			used by the platform.
			See: ViewPlatform.setOrientation
		STEPDISTANCE -- relative distance that each forward/
			backward/left/right step should move the camera,
			single float value.
		TURNANGLE -- relative rotational distance that each
			"turn" should rotate the camera, single float
			value in radians.
	"""
	platform = None
	slider = None
	initialPosition = (0,0,10)
	initialOrientation = (0,1,0,0)
	def getViewPlatform( self ):
		"""Customization Point: Instantiate ViewPlatform for this context

		The default implementation is to instantiate a
		viewplatform.ViewPlatform with position equal to
		self.initialPosition and orientation equal to
		self.initialOrientation.

		See:
			OpenGLContext.shadow.shadowcontext for
			example where this method is overridden
		"""
		if not self.platform:
			width,height = self.getViewPort()
			if width==0 or height==0:
				aspect = 1.0
			else:
				aspect = float(width)/float(height)
			self.platform = viewplatform.ViewPlatform(
				position = self.initialPosition,
				orientation = self.initialOrientation,
				aspect = aspect,
			)
		return self.platform
	def Viewpoint( self, mode = None):
		"""Customization point: Sets up the projection matrix

		This implementation potentially instantiates the
		view platform object, and then calls the object's
		render method with the mode as argument.
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.render(
			mode = mode,
		)
	def setupDefaultEventCallbacks( self, ):
		"""Customization point: Setup application default callbacks

		This method binds a large number of callbacks which support
		the OpenGLContext default camera-manipulation modes.  In
		particular:
			* unmodified arrow keys for x,z (in camera coordinate
				space) movement
			* Alt+arrow keys for x,y (in camera coordinate space)
				movement
			* Ctrl+up/down arrow keys for rotating the head backward/
				forward
			* Mouse-button-2 (right) for entering "examine" mode
			* '-' for straightening the view platform
		"""
		# left mouse activate callback to the "standard" method for interpretation
		# XXX this should all be using a generic-action: user-specific setup mapping :(
		self.addEventHandler( 'keyboard', name='<up>', state=1, modifiers=(0,0,0), function=self.forward )
		self.addEventHandler( 'keyboard', name='<up>', state=1, modifiers=(0,0,1), function=self.up )
		self.addEventHandler( 'keyboard', name='<up>', state=1, modifiers=(0,1,0), function=self.turnup)
		self.addEventHandler( 'keyboard', name='<down>', state=1, modifiers=(0,0,0), function=self.backward )
		self.addEventHandler( 'keyboard', name='<down>', state=1, modifiers=(0,0,1), function=self.down )
		self.addEventHandler( 'keyboard', name='<down>', state=1, modifiers=(0,1,0), function=self.turndown)
		
		self.addEventHandler( 'keyboard', name='<left>', state=1, modifiers=(0,0,0), function=self.turnleft)
		self.addEventHandler( 'keyboard', name='<left>', state=1, modifiers=(0,0,1), function=self.left)
		self.addEventHandler( 'keyboard', name='<right>', state=1, modifiers=(0,0,0), function=self.turnright)
		self.addEventHandler( 'keyboard', name='<right>', state=1, modifiers=(0,0,1), function=self.right)
		
		self.addEventHandler( 'keypress', name='-', modifiers=(0,0,0), function=self.straighten)

		self.addEventHandler( 'mousebutton', button=1, state = 1, modifiers=(0,0,0), function=self.startExamineMode)


	def startExamineMode (self, event):
		"""(callback) Create an examine mode interaction manager

		This callback creates an instance of
		examinemanager.ExamineManager, which will manage
		the user interaction during an "examination" of
		the scene.

		XXX
			Currently the "center" determination is broken,
			so rotation tends to occur around random points.
			I haven't yet figured out why :(
		"""
		from OpenGLContext.events import examinemanager
		try:
			center = event.unproject()
		except ValueError:
			if not self.platform:
				self.platform = self.getViewPlatform()
			center = self.platform.quaternion * [0,0,-10,0] + self.platform.position
		if not self.platform:
			self.platform = self.getViewPlatform()
		examinemanager.ExamineManager(
			self, self.platform, center,event,
		)
		
	### Eventually should support this mode (likely as a stand-alone manager)
##	def startSliderMode( self, startX,startY, buttonID, yPlane=None, **namedarguments ):
##		"""Enter "examine" mode, where a trackball is used to
##		manipulate the viewpoint so that it orbits "center"."""
##		self.slider = 1
##		self.registerCallback( None, (buttonID, 2), self.sliderUpdate )
##		self.registerCallback( None, (buttonID, 0), self.sliderRelease )
##	def sliderUpdate( self, x,y, **namedarguments ):
##		assert self.slider, '''Slider update called when no slider active!'''
##		self.triggerRedraw(1)
##	def sliderRelease( self, buttonID, **namedarguments ):
##		assert self.slider, '''Slider release called when no slider active!'''
##		self.slider = None
##		self.registerCallback( None, (buttonID, 2), None )
##		self.registerCallback( None, (buttonID, 0), None )
			
	def ViewPort( self, width, height ):
		"""Set the size of the OpenGL rendering viewport for the context

		Because the ViewPlatform provide support for
		"constant aspect ratio" in scenes, it is necessary
		to keep the ViewPlatform updated regarding the current
		aspect ratio of the ViewPort.  This implementation
		merely calls the platform's setViewport, then
		calls the super-class ViewPort method.

		XXX
			Unfortunately, because Context objects may be
			old-style classes, we can't use super(), so
			this implementation actually calls
			context.Context.ViewPort directly.
		"""
		if self.platform:
			self.platform.setViewport( width, height or 1)
		### this is ugly for a mix-in class :( 
		context.Context.ViewPort( self, width, height )

	STEPDISTANCE = 0.25
	TURNANGLE = math.pi/32
	def forward( self, event ):
		"""(callback) Move platform forward by STEPDISTANCE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.moveRelative( z = -self.STEPDISTANCE )
		self.triggerRedraw(1)
	def backward( self, event ):
		"""(callback) Move platform backward by STEPDISTANCE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.moveRelative( z = self.STEPDISTANCE )
		self.triggerRedraw(1)
	def up( self, event ):
		"""(callback) Move platform upward by STEPDISTANCE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.moveRelative( y = self.STEPDISTANCE )
		self.triggerRedraw(1)
	def down( self, event ):
		"""(callback) Move platform downward by STEPDISTANCE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.moveRelative( y = -self.STEPDISTANCE )
		self.triggerRedraw(1)
	def left( self, event ):
		"""(callback) Move platform left by STEPDISTANCE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.moveRelative( x = -self.STEPDISTANCE )
		self.triggerRedraw(1)
	def right( self, event ):
		"""(callback) Move platform right by STEPDISTANCE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.moveRelative( x = self.STEPDISTANCE )
		self.triggerRedraw(1)
	def turnup( self, event ):
		"""(callback) Rotates "head" backward (looks upward) by TURNANGLE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.turn( (1.0,0.0,0.0, self.TURNANGLE) )
		self.triggerRedraw(1)
	def turndown( self, event ):
		"""(callback) Rotates "head" forward (looks downward) by TURNANGLE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.turn( (1.0,0.0,0.0, -self.TURNANGLE) )
		self.triggerRedraw(1)
	def turnleft( self, event ):
		"""(callback) Rotates "head" to the left by TURNANGLE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.turn( (0.0,1.0,0.0, self.TURNANGLE) )
		self.triggerRedraw(1)
	def turnright( self, event ):
		"""(callback) Rotates "head" to the right by TURNANGLE

		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.turn( (0.0,1.0,0.0, -self.TURNANGLE) )
		self.triggerRedraw(1)
	def straighten( self, event ):
		"""(callback) Straightens the platform orientation

		Attempts to make the orientation equal to the
		y-axis orientation of the current orientation.

		In other words, tries to make the camera's horizon
		equal to the object-space horizon (the x,z plane)
		without altering the y-axis orientation.

		See:
			OpenGLContext.viewplatform.ViewPlatform.straighten
		
		triggers redraw after completion
		"""
		if not self.platform:
			self.platform = self.getViewPlatform()
		self.platform.straighten()
		self.triggerRedraw(1)
